@@ -136,7 +136,7 @@ import mimetypes
 rfc822_fmt = '%a, %d %b %Y %H:%M:%S GMT'
 iso8601_fmt = '%Y-%m-%dT%H:%M:%S.000Z'
 
-__version__ = "0.4.1"
+__version__ = "0.5"
 
 __all__ = "S3File", "S3Bucket", "S3Error"
 
@@ -456,8 +456,8 @@ class S3Bucket(object):
         return rv
 
     def put(self, key, data=None, acl=None, metadata={}, mimetype=None,
-            transformer=None):
-        headers = {}
+            transformer=None, headers={}):
+        headers = headers.copy()
         headers.update({"Content-Type": mimetype or guess_mimetype(key)})
         headers.update(metadata_headers(metadata))
         if acl: headers["X-AMZ-ACL"] = acl
@@ -481,6 +481,28 @@ class S3Bucket(object):
         else:
             raise S3Error.from_urllib(resp)
 
+    # TODO Expose the conditional headers, x-amz-copy-source-if-*
+    # TODO Add module-level documentation and doctests.
+    def copy(self, source, key, acl=None, metadata=None,
+             mimetype=None, headers={}):
+        """Copy S3 file *source* on format '<bucket>/<key>' to *key*.
+
+        If metadata is not None, replaces the metadata with given metadata,
+        otherwise copies the previous metadata.
+
+        Note that *acl* is not copied, but set to *private* by S3 if not given.
+        """
+        headers = headers.copy()
+        headers.update({"Content-Type": mimetype or guess_mimetype(key)})
+        headers["X-AMZ-Copy-Source"] = source
+        if acl: headers["X-AMZ-ACL"] = acl
+        if metadata is not None:
+            headers["X-AMZ-Metadata-Directive"] = "REPLACE"
+            headers.update(metadata_headers(metadata))
+        else:
+            headers["X-AMZ-Metadata-Directive"] = "COPY"
+        self.make_request("PUT", key=key, headers=headers).close()
+
     def listdir(self, prefix=None, marker=None, limit=None, delimiter=None):
         """List contents of bucket.
 
@@ -517,7 +539,7 @@ class S3Bucket(object):
 
     def url_for(self, key, authenticated=False,
                 expire=datetime.timedelta(minutes=5)):
-        """Produces the URL for given S3 object key.
+        """Produce the URL for given S3 object key.
 
         *key* specifies the S3 object path relative to the
             base URL of the bucket.
